"""SMU Model 2657A device driver module."""

import pyvisa as visa

from tm_devices.commands import SMU2657AMixin
from tm_devices.drivers.pi.source_measure_units.smu2600.smu2650a import SMU2650A
from tm_devices.helpers import DeviceConfigEntry


class SMU2657A(SMU2657AMixin, SMU2650A):
    """SMU2657A device driver."""

    ################################################################################################
    # Magic Methods
    ################################################################################################
    def __init__(
        self,
        config_entry: DeviceConfigEntry,
        verbose: bool,
        visa_resource: visa.resources.MessageBasedResource,
    ) -> None:
        """Create a SMU2657A device.

        Args:
            config_entry: A config entry object parsed by the DMConfigParser.
            verbose: A boolean indicating if verbose output should be printed.
            visa_resource: The VISA resource object.
        """
        # NOTE: This method must be defined for the documentation to properly generate
        super().__init__(config_entry, verbose, visa_resource)



"""The mmemory commands module.

These commands are used in the following models:
AFG3K, AFG3KB, AFG3KC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - MMEMory:CATalog?
    - MMEMory:CDIRectory [<directory_name>]
    - MMEMory:CDIRectory?
    - MMEMory:DELete <file_name>
    - MMEMory:LOAD:STATe {0|1|2|3|4},<file_name>
    - MMEMory:LOAD:TRACe EMEMory|EMEMory[1]|EMEMory2,<file_name>
    - MMEMory:LOCK:STATe <file_name>,{ON|OFF|<NR1>}<file_name>?
    - MMEMory:MDIRectory <directory_name>
    - MMEMory:STORe:STATe {0|1|2|3|4},<file_name>
    - MMEMory:STORe:TRACe EMEMory[1]|EMEMory2,<file_name>
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class MmemoryStoreTrace(SCPICmdWrite):
    """The ``MMEMory:STORe:TRACe`` command.

    Description:
        - This command copies a waveform data file in the Edit Memory to a file in the mass storage
          system. If the file in the mass storage is locked, this command causes an error. You
          cannot create a new file if the directory is locked.

    Usage:
        - Using the ``.write(value)`` method will send the ``MMEMory:STORe:TRACe value`` command.

    SCPI Syntax:
        ```
        - MMEMory:STORe:TRACe EMEMory[1]|EMEMory2,<file_name>
        ```
    """


class MmemoryStoreState(SCPICmdWrite):
    """The ``MMEMory:STORe:STATe`` command.

    Description:
        - This command copies a setup file in the setup memory to a specified file in the mass
          storage system. If the specified file in the mass storage system is locked, this command
          causes an error. You cannot create a new file if the directory is locked. If the setup
          memory is deleted, this command causes an error. The ``<file_name>`` argument is a quoted
          string that defines the file name and path.

    Usage:
        - Using the ``.write(value)`` method will send the ``MMEMory:STORe:STATe value`` command.

    SCPI Syntax:
        ```
        - MMEMory:STORe:STATe {0|1|2|3|4},<file_name>
        ```
    """


class MmemoryStore(SCPICmdRead):
    """The ``MMEMory:STORe`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``MMEMory:STORe?`` query.
        - Using the ``.verify(value)`` method will send the ``MMEMory:STORe?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.state``: The ``MMEMory:STORe:STATe`` command.
        - ``.trace``: The ``MMEMory:STORe:TRACe`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._state = MmemoryStoreState(device, f"{self._cmd_syntax}:STATe")
        self._trace = MmemoryStoreTrace(device, f"{self._cmd_syntax}:TRACe")

    @property
    def state(self) -> MmemoryStoreState:
        """Return the ``MMEMory:STORe:STATe`` command.

        Description:
            - This command copies a setup file in the setup memory to a specified file in the mass
              storage system. If the specified file in the mass storage system is locked, this
              command causes an error. You cannot create a new file if the directory is locked. If
              the setup memory is deleted, this command causes an error. The ``<file_name>``
              argument is a quoted string that defines the file name and path.

        Usage:
            - Using the ``.write(value)`` method will send the ``MMEMory:STORe:STATe value``
              command.

        SCPI Syntax:
            ```
            - MMEMory:STORe:STATe {0|1|2|3|4},<file_name>
            ```
        """
        return self._state

    @property
    def trace(self) -> MmemoryStoreTrace:
        """Return the ``MMEMory:STORe:TRACe`` command.

        Description:
            - This command copies a waveform data file in the Edit Memory to a file in the mass
              storage system. If the file in the mass storage is locked, this command causes an
              error. You cannot create a new file if the directory is locked.

        Usage:
            - Using the ``.write(value)`` method will send the ``MMEMory:STORe:TRACe value``
              command.

        SCPI Syntax:
            ```
            - MMEMory:STORe:TRACe EMEMory[1]|EMEMory2,<file_name>
            ```
        """
        return self._trace


class MmemoryMdirectory(SCPICmdWrite):
    """The ``MMEMory:MDIRectory`` command.

    Description:
        - This command creates a directory in the mass storage system. If the specified directory is
          locked in the mass storage system, this command causes an error.

    Usage:
        - Using the ``.write(value)`` method will send the ``MMEMory:MDIRectory value`` command.

    SCPI Syntax:
        ```
        - MMEMory:MDIRectory <directory_name>
        ```
    """


class MmemoryLockState(SCPICmdWrite):
    """The ``MMEMory:LOCK:STATe`` command.

    Description:
        - This command sets or queries whether to lock a file or directory in the mass storage
          system. If you lock a file or directory, you cannot overwrite or delete it.

    Usage:
        - Using the ``.write(value)`` method will send the ``MMEMory:LOCK:STATe value`` command.

    SCPI Syntax:
        ```
        - MMEMory:LOCK:STATe <file_name>,{ON|OFF|<NR1>}<file_name>?
        ```
    """


class MmemoryLock(SCPICmdRead):
    """The ``MMEMory:LOCK`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``MMEMory:LOCK?`` query.
        - Using the ``.verify(value)`` method will send the ``MMEMory:LOCK?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.state``: The ``MMEMory:LOCK:STATe`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._state = MmemoryLockState(device, f"{self._cmd_syntax}:STATe")

    @property
    def state(self) -> MmemoryLockState:
        """Return the ``MMEMory:LOCK:STATe`` command.

        Description:
            - This command sets or queries whether to lock a file or directory in the mass storage
              system. If you lock a file or directory, you cannot overwrite or delete it.

        Usage:
            - Using the ``.write(value)`` method will send the ``MMEMory:LOCK:STATe value`` command.

        SCPI Syntax:
            ```
            - MMEMory:LOCK:STATe <file_name>,{ON|OFF|<NR1>}<file_name>?
            ```
        """
        return self._state


class MmemoryLoadTrace(SCPICmdWrite):
    """The ``MMEMory:LOAD:TRACe`` command.

    Description:
        - This command copies a waveform data file in the mass storage system to Edit Memory. If the
          file format is different, this command causes an error.

    Usage:
        - Using the ``.write(value)`` method will send the ``MMEMory:LOAD:TRACe value`` command.

    SCPI Syntax:
        ```
        - MMEMory:LOAD:TRACe EMEMory|EMEMory[1]|EMEMory2,<file_name>
        ```
    """


class MmemoryLoadState(SCPICmdWrite):
    """The ``MMEMory:LOAD:STATe`` command.

    Description:
        - This command copies a setup file in the mass storage system to an internal setup memory.
          If a specified internal setup memory is locked, this command causes an error. When you
          power off the instrument, the setups are automatically overwritten in the setup memory 0
          (last setup memory).

    Usage:
        - Using the ``.write(value)`` method will send the ``MMEMory:LOAD:STATe value`` command.

    SCPI Syntax:
        ```
        - MMEMory:LOAD:STATe {0|1|2|3|4},<file_name>
        ```
    """


class MmemoryLoad(SCPICmdRead):
    """The ``MMEMory:LOAD`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``MMEMory:LOAD?`` query.
        - Using the ``.verify(value)`` method will send the ``MMEMory:LOAD?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.state``: The ``MMEMory:LOAD:STATe`` command.
        - ``.trace``: The ``MMEMory:LOAD:TRACe`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._state = MmemoryLoadState(device, f"{self._cmd_syntax}:STATe")
        self._trace = MmemoryLoadTrace(device, f"{self._cmd_syntax}:TRACe")

    @property
    def state(self) -> MmemoryLoadState:
        """Return the ``MMEMory:LOAD:STATe`` command.

        Description:
            - This command copies a setup file in the mass storage system to an internal setup
              memory. If a specified internal setup memory is locked, this command causes an error.
              When you power off the instrument, the setups are automatically overwritten in the
              setup memory 0 (last setup memory).

        Usage:
            - Using the ``.write(value)`` method will send the ``MMEMory:LOAD:STATe value`` command.

        SCPI Syntax:
            ```
            - MMEMory:LOAD:STATe {0|1|2|3|4},<file_name>
            ```
        """
        return self._state

    @property
    def trace(self) -> MmemoryLoadTrace:
        """Return the ``MMEMory:LOAD:TRACe`` command.

        Description:
            - This command copies a waveform data file in the mass storage system to Edit Memory. If
              the file format is different, this command causes an error.

        Usage:
            - Using the ``.write(value)`` method will send the ``MMEMory:LOAD:TRACe value`` command.

        SCPI Syntax:
            ```
            - MMEMory:LOAD:TRACe EMEMory|EMEMory[1]|EMEMory2,<file_name>
            ```
        """
        return self._trace


class MmemoryDelete(SCPICmdWrite):
    """The ``MMEMory:DELete`` command.

    Description:
        - This command deletes a file or directory from the mass storage system. If a specified file
          in the mass storage is not allowed to overwrite or delete, this command causes an error.
          You can delete a directory if it is empty.

    Usage:
        - Using the ``.write(value)`` method will send the ``MMEMory:DELete value`` command.

    SCPI Syntax:
        ```
        - MMEMory:DELete <file_name>
        ```
    """


class MmemoryCdirectory(SCPICmdWrite, SCPICmdRead):
    """The ``MMEMory:CDIRectory`` command.

    Description:
        - This command changes the current working directory in the mass storage system.

    Usage:
        - Using the ``.query()`` method will send the ``MMEMory:CDIRectory?`` query.
        - Using the ``.verify(value)`` method will send the ``MMEMory:CDIRectory?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MMEMory:CDIRectory value`` command.

    SCPI Syntax:
        ```
        - MMEMory:CDIRectory [<directory_name>]
        - MMEMory:CDIRectory?
        ```
    """


class MmemoryCatalog(SCPICmdRead):
    """The ``MMEMory:CATalog`` command.

    Description:
        - This query-only command returns the current state of the mass storage system (USB memory).

    Usage:
        - Using the ``.query()`` method will send the ``MMEMory:CATalog?`` query.
        - Using the ``.verify(value)`` method will send the ``MMEMory:CATalog?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MMEMory:CATalog?
        ```
    """


class Mmemory(SCPICmdRead):
    """The ``MMEMory`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``MMEMory?`` query.
        - Using the ``.verify(value)`` method will send the ``MMEMory?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.catalog``: The ``MMEMory:CATalog`` command.
        - ``.cdirectory``: The ``MMEMory:CDIRectory`` command.
        - ``.delete``: The ``MMEMory:DELete`` command.
        - ``.load``: The ``MMEMory:LOAD`` command tree.
        - ``.lock``: The ``MMEMory:LOCK`` command tree.
        - ``.mdirectory``: The ``MMEMory:MDIRectory`` command.
        - ``.store``: The ``MMEMory:STORe`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "MMEMory") -> None:
        super().__init__(device, cmd_syntax)
        self._catalog = MmemoryCatalog(device, f"{self._cmd_syntax}:CATalog")
        self._cdirectory = MmemoryCdirectory(device, f"{self._cmd_syntax}:CDIRectory")
        self._delete = MmemoryDelete(device, f"{self._cmd_syntax}:DELete")
        self._load = MmemoryLoad(device, f"{self._cmd_syntax}:LOAD")
        self._lock = MmemoryLock(device, f"{self._cmd_syntax}:LOCK")
        self._mdirectory = MmemoryMdirectory(device, f"{self._cmd_syntax}:MDIRectory")
        self._store = MmemoryStore(device, f"{self._cmd_syntax}:STORe")

    @property
    def catalog(self) -> MmemoryCatalog:
        """Return the ``MMEMory:CATalog`` command.

        Description:
            - This query-only command returns the current state of the mass storage system (USB
              memory).

        Usage:
            - Using the ``.query()`` method will send the ``MMEMory:CATalog?`` query.
            - Using the ``.verify(value)`` method will send the ``MMEMory:CATalog?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MMEMory:CATalog?
            ```
        """
        return self._catalog

    @property
    def cdirectory(self) -> MmemoryCdirectory:
        """Return the ``MMEMory:CDIRectory`` command.

        Description:
            - This command changes the current working directory in the mass storage system.

        Usage:
            - Using the ``.query()`` method will send the ``MMEMory:CDIRectory?`` query.
            - Using the ``.verify(value)`` method will send the ``MMEMory:CDIRectory?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``MMEMory:CDIRectory value`` command.

        SCPI Syntax:
            ```
            - MMEMory:CDIRectory [<directory_name>]
            - MMEMory:CDIRectory?
            ```
        """
        return self._cdirectory

    @property
    def delete(self) -> MmemoryDelete:
        """Return the ``MMEMory:DELete`` command.

        Description:
            - This command deletes a file or directory from the mass storage system. If a specified
              file in the mass storage is not allowed to overwrite or delete, this command causes an
              error. You can delete a directory if it is empty.

        Usage:
            - Using the ``.write(value)`` method will send the ``MMEMory:DELete value`` command.

        SCPI Syntax:
            ```
            - MMEMory:DELete <file_name>
            ```
        """
        return self._delete

    @property
    def load(self) -> MmemoryLoad:
        """Return the ``MMEMory:LOAD`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``MMEMory:LOAD?`` query.
            - Using the ``.verify(value)`` method will send the ``MMEMory:LOAD?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.state``: The ``MMEMory:LOAD:STATe`` command.
            - ``.trace``: The ``MMEMory:LOAD:TRACe`` command.
        """
        return self._load

    @property
    def lock(self) -> MmemoryLock:
        """Return the ``MMEMory:LOCK`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``MMEMory:LOCK?`` query.
            - Using the ``.verify(value)`` method will send the ``MMEMory:LOCK?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.state``: The ``MMEMory:LOCK:STATe`` command.
        """
        return self._lock

    @property
    def mdirectory(self) -> MmemoryMdirectory:
        """Return the ``MMEMory:MDIRectory`` command.

        Description:
            - This command creates a directory in the mass storage system. If the specified
              directory is locked in the mass storage system, this command causes an error.

        Usage:
            - Using the ``.write(value)`` method will send the ``MMEMory:MDIRectory value`` command.

        SCPI Syntax:
            ```
            - MMEMory:MDIRectory <directory_name>
            ```
        """
        return self._mdirectory

    @property
    def store(self) -> MmemoryStore:
        """Return the ``MMEMory:STORe`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``MMEMory:STORe?`` query.
            - Using the ``.verify(value)`` method will send the ``MMEMory:STORe?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.state``: The ``MMEMory:STORe:STATe`` command.
            - ``.trace``: The ``MMEMory:STORe:TRACe`` command.
        """
        return self._store



"""The email commands module.

These commands are used in the following models:
DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7K, DPO7KC, DSA70KC, DSA70KD, MSO5K, MSO5KB,
MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - EMail {TESt|RESET}
    - EMail:ATTempts <NR1>
    - EMail:ATTempts?
    - EMail:AUTHLogin {<QString>}
    - EMail:AUTHLogin?
    - EMail:AUTHPassword {<QString>}
    - EMail:COUNt?
    - EMail:FROm {<QString>}
    - EMail:FROm?
    - EMail:HOSTwanted {<QString>}
    - EMail:HOSTwanted?
    - EMail:IMAGe {<NR1>|ON|OFF|}
    - EMail:IMAGe?
    - EMail:LIMit {ON|OFF|<NR1>}
    - EMail:LIMit?
    - EMail:MASK {<NR1>|ON|OFF}
    - EMail:MASK?
    - EMail:MAXSize {<NR1>}
    - EMail:MAXSize?
    - EMail:MEASUrement {<NR1>|ON|OFF}
    - EMail:MEASUrement?
    - EMail:NUMEMails {<NR1>}
    - EMail:NUMEMails?
    - EMail:SMTPPort {<NR1>}
    - EMail:SMTPPort?
    - EMail:SMTPServer <QString>
    - EMail:SMTPServer?
    - EMail:STATUS?
    - EMail:TIMEOut <NR1>
    - EMail:TIMEOut?
    - EMail:TO <QString>
    - EMail:TO?
    - EMail:TRIGger {<NR1>|ON|OFF}
    - EMail:TRIGger?
    - EMail:WAVEform {<NR1>|ON|OFF}
    - EMail:WAVEform?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class EmailWaveform(SCPICmdWrite, SCPICmdRead):
    """The ``EMail:WAVEform`` command.

    Description:
        - This command sets or queries whether waveform data is included in e-mail. The waveforms to
          be included, the start and stop points (and, or, frames for Fast Frame mode), and the data
          formats are specified by the DATA and WFMOUTPRE commands within the Waveform Transfer
          Command group.

    Usage:
        - Using the ``.query()`` method will send the ``EMail:WAVEform?`` query.
        - Using the ``.verify(value)`` method will send the ``EMail:WAVEform?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``EMail:WAVEform value`` command.

    SCPI Syntax:
        ```
        - EMail:WAVEform {<NR1>|ON|OFF}
        - EMail:WAVEform?
        ```

    Info:
        - ``<NR1>`` = 0 disables the inclusion of waveform data in the e-mail; any other value
          enables including waveform data in the e-mail.
        - ``ON`` enables the inclusion of waveform data in e-mail.
        - ``OFF`` disables the inclusion of waveform data in e-mail.
    """


class EmailTrigger(SCPICmdWrite, SCPICmdRead):
    """The ``EMail:TRIGger`` command.

    Description:
        - This command sets or queries whether e-mail is sent when a trigger occurs. The e-mail is
          sent after the acquisition is complete.

    Usage:
        - Using the ``.query()`` method will send the ``EMail:TRIGger?`` query.
        - Using the ``.verify(value)`` method will send the ``EMail:TRIGger?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``EMail:TRIGger value`` command.

    SCPI Syntax:
        ```
        - EMail:TRIGger {<NR1>|ON|OFF}
        - EMail:TRIGger?
        ```

    Info:
        - ``<NR1>`` = 0 disables sending e-mail when a trigger occurs; any other value enables
          sending e-mail when a trigger occurs.
        - ``ON`` argument enables sending e-mail when a trigger occurs.
        - ``OFF`` disables sending e-mail when a trigger occurs.
    """


class EmailTo(SCPICmdWrite, SCPICmdRead):
    """The ``EMail:TO`` command.

    Description:
        - This command sets or queries the address of the recipient(s) of an e-mail.

    Usage:
        - Using the ``.query()`` method will send the ``EMail:TO?`` query.
        - Using the ``.verify(value)`` method will send the ``EMail:TO?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``EMail:TO value`` command.

    SCPI Syntax:
        ```
        - EMail:TO <QString>
        - EMail:TO?
        ```

    Info:
        - ``<QString>`` argument is the e-mail address of the recipient (or recipients). Multiple
          addresses are separated with semicolons (;). For example,
          'johndoe@tek.com;billsmith@tek.com' specifies that both johndoe and billsmith at tek.com
          will receive e-mail when the event occurs. The length of this string is limited to 252
          characters.
    """

    _WRAP_ARG_WITH_QUOTES = True


class EmailTimeout(SCPICmdWrite, SCPICmdRead):
    """The ``EMail:TIMEOut`` command.

    Description:
        - This command sets or queries the global timeout in seconds. The default is 30 seconds. You
          use this timeout for socket connections and might need to change it from the default on
          some networks.

    Usage:
        - Using the ``.query()`` method will send the ``EMail:TIMEOut?`` query.
        - Using the ``.verify(value)`` method will send the ``EMail:TIMEOut?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``EMail:TIMEOut value`` command.

    SCPI Syntax:
        ```
        - EMail:TIMEOut <NR1>
        - EMail:TIMEOut?
        ```

    Info:
        - ``<NR1>`` argument is the global timeout in seconds. The value for NR1 can be 0 through
          500.
    """


class EmailStatus(SCPICmdRead):
    """The ``EMail:STATUS`` command.

    Description:
        - This query only command returns the status of the last e-mail you attempted to send.

    Usage:
        - Using the ``.query()`` method will send the ``EMail:STATUS?`` query.
        - Using the ``.verify(value)`` method will send the ``EMail:STATUS?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - EMail:STATUS?
        ```
    """


class EmailSmtpserver(SCPICmdWrite, SCPICmdRead):
    """The ``EMail:SMTPServer`` command.

    Description:
        - This command sets or queries the address of the SMTP mail server.

    Usage:
        - Using the ``.query()`` method will send the ``EMail:SMTPServer?`` query.
        - Using the ``.verify(value)`` method will send the ``EMail:SMTPServer?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``EMail:SMTPServer value`` command.

    SCPI Syntax:
        ```
        - EMail:SMTPServer <QString>
        - EMail:SMTPServer?
        ```

    Info:
        - ``<QString>`` argument is the address of the SMTP mail server that will handle the mail
          service request. For example, 'smtp.tek.com'.
    """

    _WRAP_ARG_WITH_QUOTES = True


class EmailSmtpport(SCPICmdWrite, SCPICmdRead):
    """The ``EMail:SMTPPort`` command.

    Description:
        - This command sets or queries the SMTP port number that the e-mail server uses if other
          than the default of 25.

    Usage:
        - Using the ``.query()`` method will send the ``EMail:SMTPPort?`` query.
        - Using the ``.verify(value)`` method will send the ``EMail:SMTPPort?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``EMail:SMTPPort value`` command.

    SCPI Syntax:
        ```
        - EMail:SMTPPort {<NR1>}
        - EMail:SMTPPort?
        ```

    Info:
        - ``<NR1>`` can be 1 through 65535. This number specifies the TCPIP port number.
    """


class EmailNumemails(SCPICmdWrite, SCPICmdRead):
    """The ``EMail:NUMEMails`` command.

    Description:
        - This command sets or queries the number of e-mails you can send when Email on Event is
          armed, from 1 to 50. This limit is to restrict e-mail floods.

    Usage:
        - Using the ``.query()`` method will send the ``EMail:NUMEMails?`` query.
        - Using the ``.verify(value)`` method will send the ``EMail:NUMEMails?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``EMail:NUMEMails value`` command.

    SCPI Syntax:
        ```
        - EMail:NUMEMails {<NR1>}
        - EMail:NUMEMails?
        ```

    Info:
        - ``<NR1>`` can be 1 through 50. This is the number of e-mails you can send before the
          number of sent e-mails must be reset with the EMail RESet command.
    """


class EmailMeasurement(SCPICmdWrite, SCPICmdRead):
    """The ``EMail:MEASUrement`` command.

    Description:
        - This command sets or queries whether measurement data is included as an attachment to
          e-mail. You must specify the saved measurement selection and the measurement format using
          the Email on Event setup menu. There are no remote commands for this purpose.

    Usage:
        - Using the ``.query()`` method will send the ``EMail:MEASUrement?`` query.
        - Using the ``.verify(value)`` method will send the ``EMail:MEASUrement?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``EMail:MEASUrement value`` command.

    SCPI Syntax:
        ```
        - EMail:MEASUrement {<NR1>|ON|OFF}
        - EMail:MEASUrement?
        ```

    Info:
        - ``<NR1>`` = 0 disables the inclusion of measurement data in the e-mail; any other value
          enables including measurement data in the e-mail.
        - ``ON`` enables the inclusion of measurement data in e-mail.
        - ``OFF`` disables the inclusion of measurement data in e-mail.
    """


class EmailMaxsize(SCPICmdWrite, SCPICmdRead):
    """The ``EMail:MAXSize`` command.

    Description:
        - This command sets or queries the maximum size (in megabytes) of e-mail that can be sent to
          the SMTP server.

    Usage:
        - Using the ``.query()`` method will send the ``EMail:MAXSize?`` query.
        - Using the ``.verify(value)`` method will send the ``EMail:MAXSize?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``EMail:MAXSize value`` command.

    SCPI Syntax:
        ```
        - EMail:MAXSize {<NR1>}
        - EMail:MAXSize?
        ```

    Info:
        - ``<NR1>`` can be 0 through 2000 (megabytes). This is the maximum size of each e-mail that
          can be sent to the SMTP server.
    """


class EmailMask(SCPICmdWrite, SCPICmdRead):
    """The ``EMail:MASK`` command.

    Description:
        - This command sets or queries whether e-mail is sent when a mask test failure occurs.

    Usage:
        - Using the ``.query()`` method will send the ``EMail:MASK?`` query.
        - Using the ``.verify(value)`` method will send the ``EMail:MASK?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``EMail:MASK value`` command.

    SCPI Syntax:
        ```
        - EMail:MASK {<NR1>|ON|OFF}
        - EMail:MASK?
        ```

    Info:
        - ``<NR1>`` = 0 disables sending e-mail when a mask test failure occurs; any other value
          enables sending e-mail when a mask test failure occurs.
        - ``ON`` argument enables sending e-mail when a mask test failure occurs.
        - ``OFF`` disables sending e-mail when a mask test failure occurs.
    """


class EmailLimit(SCPICmdWrite, SCPICmdRead):
    """The ``EMail:LIMit`` command.

    Description:
        - This command sets or queries whether e-mail is sent when a limit test failure occurs. This
          command is the same as the ``LIMit:EMail`` command.

    Usage:
        - Using the ``.query()`` method will send the ``EMail:LIMit?`` query.
        - Using the ``.verify(value)`` method will send the ``EMail:LIMit?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``EMail:LIMit value`` command.

    SCPI Syntax:
        ```
        - EMail:LIMit {ON|OFF|<NR1>}
        - EMail:LIMit?
        ```

    Info:
        - ``ON`` argument enables sending e-mail when a limit test failure occurs.
        - ``OFF`` disables sending e-mail when a limit test failure occurs.
        - ``<NR1>`` = 0 disables sending e-mail when a limit test failure occurs; any other value
          enables sending e-mail when a limit test failure occurs.
    """


class EmailImage(SCPICmdWrite, SCPICmdRead):
    """The ``EMail:IMAGe`` command.

    Description:
        - This command sets or queries whether a screen image is included in e-mail. If this command
          is set to ON, the format and content of the screen image included is specified using the
          EXPort commands within the Hard Copy Command group.

    Usage:
        - Using the ``.query()`` method will send the ``EMail:IMAGe?`` query.
        - Using the ``.verify(value)`` method will send the ``EMail:IMAGe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``EMail:IMAGe value`` command.

    SCPI Syntax:
        ```
        - EMail:IMAGe {<NR1>|ON|OFF|}
        - EMail:IMAGe?
        ```

    Info:
        - ``<NR1>`` = 0 disables the inclusion of a screen image in e-mail; any other value enables
          the inclusion of a screen image in e-mail.
        - ``ON`` argument enables the inclusion of a screen image in e-mail.
        - ``OFF`` argument disables the inclusion of a screen image in e-mail.
    """


class EmailHostwanted(SCPICmdWrite, SCPICmdRead):
    """The ``EMail:HOSTwanted`` command.

    Description:
        - This command sets or queries the host name that will be used when e-mail is sent to the
          SMTP e-mail server if the DPO host name will not work.

    Usage:
        - Using the ``.query()`` method will send the ``EMail:HOSTwanted?`` query.
        - Using the ``.verify(value)`` method will send the ``EMail:HOSTwanted?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``EMail:HOSTwanted value`` command.

    SCPI Syntax:
        ```
        - EMail:HOSTwanted {<QString>}
        - EMail:HOSTwanted?
        ```

    Info:
        - ``<QString>`` argument is a string that specifies the host name to be used with the SMTP
          e-mail server if the default DPO host name will not work.
    """


class EmailFrom(SCPICmdWrite, SCPICmdRead):
    """The ``EMail:FROm`` command.

    Description:
        - This command sets or queries the From line in the e-mail. The default for the From line is
          the instrument model and serial number.

    Usage:
        - Using the ``.query()`` method will send the ``EMail:FROm?`` query.
        - Using the ``.verify(value)`` method will send the ``EMail:FROm?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``EMail:FROm value`` command.

    SCPI Syntax:
        ```
        - EMail:FROm {<QString>}
        - EMail:FROm?
        ```

    Info:
        - ``<QString>`` argument is a string that is placed in the From line of the e-mail. An
          example is 'johnz'.
    """


class EmailCount(SCPICmdRead):
    """The ``EMail:COUNt`` command.

    Description:
        - This query only command returns the number of e-mails that have been sent since Email on
          Event was armed. The range of values returned can be from 0 to 50.

    Usage:
        - Using the ``.query()`` method will send the ``EMail:COUNt?`` query.
        - Using the ``.verify(value)`` method will send the ``EMail:COUNt?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - EMail:COUNt?
        ```
    """


class EmailAuthpassword(SCPICmdWrite):
    """The ``EMail:AUTHPassword`` command.

    Description:
        - This command (no query form) sets the password that will be used if the SMTP e-mail server
          requires one for authentication.

    Usage:
        - Using the ``.write(value)`` method will send the ``EMail:AUTHPassword value`` command.

    SCPI Syntax:
        ```
        - EMail:AUTHPassword {<QString>}
        ```

    Info:
        - ``<QString>`` argument is a string that specifies the password to be used with the SMTP
          e-mail server.
    """


class EmailAuthlogin(SCPICmdWrite, SCPICmdRead):
    """The ``EMail:AUTHLogin`` command.

    Description:
        - This command sets or queries the login name that will be used if the SMTP e-mail server
          requires one for authentication.

    Usage:
        - Using the ``.query()`` method will send the ``EMail:AUTHLogin?`` query.
        - Using the ``.verify(value)`` method will send the ``EMail:AUTHLogin?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``EMail:AUTHLogin value`` command.

    SCPI Syntax:
        ```
        - EMail:AUTHLogin {<QString>}
        - EMail:AUTHLogin?
        ```

    Info:
        - ``<QString>`` argument is a string that specifies the login name to be used with the SMTP
          e-mail server.
    """


class EmailAttempts(SCPICmdWrite, SCPICmdRead):
    """The ``EMail:ATTempts`` command.

    Description:
        - This command sets or queries the number of times that an attempt will be made to send
          e-mail to the SMTP e-mail server. The default is 1.

    Usage:
        - Using the ``.query()`` method will send the ``EMail:ATTempts?`` query.
        - Using the ``.verify(value)`` method will send the ``EMail:ATTempts?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``EMail:ATTempts value`` command.

    SCPI Syntax:
        ```
        - EMail:ATTempts <NR1>
        - EMail:ATTempts?
        ```

    Info:
        - ``<NR1>`` can be 1 through 20. This number is the specified number of attempts that will
          be made to send e-mail to the SMTP server.
    """


#  pylint: disable=too-many-instance-attributes
class Email(SCPICmdWrite, SCPICmdRead):
    """The ``EMail`` command.

    Description:
        - This command (no query form) sends a test e-mail message or sets the current e-mail sent
          count to zero.

    Usage:
        - Using the ``.write(value)`` method will send the ``EMail value`` command.

    SCPI Syntax:
        ```
        - EMail {TESt|RESET}
        ```

    Info:
        - ``TESt`` argument sends a test e-mail message.
        - ``RESET`` argument sets the e-mail sent count to zero.

    Properties:
        - ``.attempts``: The ``EMail:ATTempts`` command.
        - ``.authlogin``: The ``EMail:AUTHLogin`` command.
        - ``.authpassword``: The ``EMail:AUTHPassword`` command.
        - ``.count``: The ``EMail:COUNt`` command.
        - ``.from``: The ``EMail:FROm`` command.
        - ``.hostwanted``: The ``EMail:HOSTwanted`` command.
        - ``.image``: The ``EMail:IMAGe`` command.
        - ``.limit``: The ``EMail:LIMit`` command.
        - ``.mask``: The ``EMail:MASK`` command.
        - ``.maxsize``: The ``EMail:MAXSize`` command.
        - ``.measurement``: The ``EMail:MEASUrement`` command.
        - ``.numemails``: The ``EMail:NUMEMails`` command.
        - ``.smtpport``: The ``EMail:SMTPPort`` command.
        - ``.smtpserver``: The ``EMail:SMTPServer`` command.
        - ``.status``: The ``EMail:STATUS`` command.
        - ``.timeout``: The ``EMail:TIMEOut`` command.
        - ``.to``: The ``EMail:TO`` command.
        - ``.trigger``: The ``EMail:TRIGger`` command.
        - ``.waveform``: The ``EMail:WAVEform`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "EMail") -> None:
        super().__init__(device, cmd_syntax)
        self._attempts = EmailAttempts(device, f"{self._cmd_syntax}:ATTempts")
        self._authlogin = EmailAuthlogin(device, f"{self._cmd_syntax}:AUTHLogin")
        self._authpassword = EmailAuthpassword(device, f"{self._cmd_syntax}:AUTHPassword")
        self._count = EmailCount(device, f"{self._cmd_syntax}:COUNt")
        self._from = EmailFrom(device, f"{self._cmd_syntax}:FROm")
        self._hostwanted = EmailHostwanted(device, f"{self._cmd_syntax}:HOSTwanted")
        self._image = EmailImage(device, f"{self._cmd_syntax}:IMAGe")
        self._limit = EmailLimit(device, f"{self._cmd_syntax}:LIMit")
        self._mask = EmailMask(device, f"{self._cmd_syntax}:MASK")
        self._maxsize = EmailMaxsize(device, f"{self._cmd_syntax}:MAXSize")
        self._measurement = EmailMeasurement(device, f"{self._cmd_syntax}:MEASUrement")
        self._numemails = EmailNumemails(device, f"{self._cmd_syntax}:NUMEMails")
        self._smtpport = EmailSmtpport(device, f"{self._cmd_syntax}:SMTPPort")
        self._smtpserver = EmailSmtpserver(device, f"{self._cmd_syntax}:SMTPServer")
        self._status = EmailStatus(device, f"{self._cmd_syntax}:STATUS")
        self._timeout = EmailTimeout(device, f"{self._cmd_syntax}:TIMEOut")
        self._to = EmailTo(device, f"{self._cmd_syntax}:TO")
        self._trigger = EmailTrigger(device, f"{self._cmd_syntax}:TRIGger")
        self._waveform = EmailWaveform(device, f"{self._cmd_syntax}:WAVEform")

    @property
    def attempts(self) -> EmailAttempts:
        """Return the ``EMail:ATTempts`` command.

        Description:
            - This command sets or queries the number of times that an attempt will be made to send
              e-mail to the SMTP e-mail server. The default is 1.

        Usage:
            - Using the ``.query()`` method will send the ``EMail:ATTempts?`` query.
            - Using the ``.verify(value)`` method will send the ``EMail:ATTempts?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``EMail:ATTempts value`` command.

        SCPI Syntax:
            ```
            - EMail:ATTempts <NR1>
            - EMail:ATTempts?
            ```

        Info:
            - ``<NR1>`` can be 1 through 20. This number is the specified number of attempts that
              will be made to send e-mail to the SMTP server.
        """
        return self._attempts

    @property
    def authlogin(self) -> EmailAuthlogin:
        """Return the ``EMail:AUTHLogin`` command.

        Description:
            - This command sets or queries the login name that will be used if the SMTP e-mail
              server requires one for authentication.

        Usage:
            - Using the ``.query()`` method will send the ``EMail:AUTHLogin?`` query.
            - Using the ``.verify(value)`` method will send the ``EMail:AUTHLogin?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``EMail:AUTHLogin value`` command.

        SCPI Syntax:
            ```
            - EMail:AUTHLogin {<QString>}
            - EMail:AUTHLogin?
            ```

        Info:
            - ``<QString>`` argument is a string that specifies the login name to be used with the
              SMTP e-mail server.
        """
        return self._authlogin

    @property
    def authpassword(self) -> EmailAuthpassword:
        """Return the ``EMail:AUTHPassword`` command.

        Description:
            - This command (no query form) sets the password that will be used if the SMTP e-mail
              server requires one for authentication.

        Usage:
            - Using the ``.write(value)`` method will send the ``EMail:AUTHPassword value`` command.

        SCPI Syntax:
            ```
            - EMail:AUTHPassword {<QString>}
            ```

        Info:
            - ``<QString>`` argument is a string that specifies the password to be used with the
              SMTP e-mail server.
        """
        return self._authpassword

    @property
    def count(self) -> EmailCount:
        """Return the ``EMail:COUNt`` command.

        Description:
            - This query only command returns the number of e-mails that have been sent since Email
              on Event was armed. The range of values returned can be from 0 to 50.

        Usage:
            - Using the ``.query()`` method will send the ``EMail:COUNt?`` query.
            - Using the ``.verify(value)`` method will send the ``EMail:COUNt?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - EMail:COUNt?
            ```
        """
        return self._count

    @property
    def from_(self) -> EmailFrom:
        """Return the ``EMail:FROm`` command.

        Description:
            - This command sets or queries the From line in the e-mail. The default for the From
              line is the instrument model and serial number.

        Usage:
            - Using the ``.query()`` method will send the ``EMail:FROm?`` query.
            - Using the ``.verify(value)`` method will send the ``EMail:FROm?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``EMail:FROm value`` command.

        SCPI Syntax:
            ```
            - EMail:FROm {<QString>}
            - EMail:FROm?
            ```

        Info:
            - ``<QString>`` argument is a string that is placed in the From line of the e-mail. An
              example is 'johnz'.
        """
        return self._from

    @property
    def hostwanted(self) -> EmailHostwanted:
        """Return the ``EMail:HOSTwanted`` command.

        Description:
            - This command sets or queries the host name that will be used when e-mail is sent to
              the SMTP e-mail server if the DPO host name will not work.

        Usage:
            - Using the ``.query()`` method will send the ``EMail:HOSTwanted?`` query.
            - Using the ``.verify(value)`` method will send the ``EMail:HOSTwanted?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``EMail:HOSTwanted value`` command.

        SCPI Syntax:
            ```
            - EMail:HOSTwanted {<QString>}
            - EMail:HOSTwanted?
            ```

        Info:
            - ``<QString>`` argument is a string that specifies the host name to be used with the
              SMTP e-mail server if the default DPO host name will not work.
        """
        return self._hostwanted

    @property
    def image(self) -> EmailImage:
        """Return the ``EMail:IMAGe`` command.

        Description:
            - This command sets or queries whether a screen image is included in e-mail. If this
              command is set to ON, the format and content of the screen image included is specified
              using the EXPort commands within the Hard Copy Command group.

        Usage:
            - Using the ``.query()`` method will send the ``EMail:IMAGe?`` query.
            - Using the ``.verify(value)`` method will send the ``EMail:IMAGe?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``EMail:IMAGe value`` command.

        SCPI Syntax:
            ```
            - EMail:IMAGe {<NR1>|ON|OFF|}
            - EMail:IMAGe?
            ```

        Info:
            - ``<NR1>`` = 0 disables the inclusion of a screen image in e-mail; any other value
              enables the inclusion of a screen image in e-mail.
            - ``ON`` argument enables the inclusion of a screen image in e-mail.
            - ``OFF`` argument disables the inclusion of a screen image in e-mail.
        """
        return self._image

    @property
    def limit(self) -> EmailLimit:
        """Return the ``EMail:LIMit`` command.

        Description:
            - This command sets or queries whether e-mail is sent when a limit test failure occurs.
              This command is the same as the ``LIMit:EMail`` command.

        Usage:
            - Using the ``.query()`` method will send the ``EMail:LIMit?`` query.
            - Using the ``.verify(value)`` method will send the ``EMail:LIMit?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``EMail:LIMit value`` command.

        SCPI Syntax:
            ```
            - EMail:LIMit {ON|OFF|<NR1>}
            - EMail:LIMit?
            ```

        Info:
            - ``ON`` argument enables sending e-mail when a limit test failure occurs.
            - ``OFF`` disables sending e-mail when a limit test failure occurs.
            - ``<NR1>`` = 0 disables sending e-mail when a limit test failure occurs; any other
              value enables sending e-mail when a limit test failure occurs.
        """
        return self._limit

    @property
    def mask(self) -> EmailMask:
        """Return the ``EMail:MASK`` command.

        Description:
            - This command sets or queries whether e-mail is sent when a mask test failure occurs.

        Usage:
            - Using the ``.query()`` method will send the ``EMail:MASK?`` query.
            - Using the ``.verify(value)`` method will send the ``EMail:MASK?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``EMail:MASK value`` command.

        SCPI Syntax:
            ```
            - EMail:MASK {<NR1>|ON|OFF}
            - EMail:MASK?
            ```

        Info:
            - ``<NR1>`` = 0 disables sending e-mail when a mask test failure occurs; any other value
              enables sending e-mail when a mask test failure occurs.
            - ``ON`` argument enables sending e-mail when a mask test failure occurs.
            - ``OFF`` disables sending e-mail when a mask test failure occurs.
        """
        return self._mask

    @property
    def maxsize(self) -> EmailMaxsize:
        """Return the ``EMail:MAXSize`` command.

        Description:
            - This command sets or queries the maximum size (in megabytes) of e-mail that can be
              sent to the SMTP server.

        Usage:
            - Using the ``.query()`` method will send the ``EMail:MAXSize?`` query.
            - Using the ``.verify(value)`` method will send the ``EMail:MAXSize?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``EMail:MAXSize value`` command.

        SCPI Syntax:
            ```
            - EMail:MAXSize {<NR1>}
            - EMail:MAXSize?
            ```

        Info:
            - ``<NR1>`` can be 0 through 2000 (megabytes). This is the maximum size of each e-mail
              that can be sent to the SMTP server.
        """
        return self._maxsize

    @property
    def measurement(self) -> EmailMeasurement:
        """Return the ``EMail:MEASUrement`` command.

        Description:
            - This command sets or queries whether measurement data is included as an attachment to
              e-mail. You must specify the saved measurement selection and the measurement format
              using the Email on Event setup menu. There are no remote commands for this purpose.

        Usage:
            - Using the ``.query()`` method will send the ``EMail:MEASUrement?`` query.
            - Using the ``.verify(value)`` method will send the ``EMail:MEASUrement?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``EMail:MEASUrement value`` command.

        SCPI Syntax:
            ```
            - EMail:MEASUrement {<NR1>|ON|OFF}
            - EMail:MEASUrement?
            ```

        Info:
            - ``<NR1>`` = 0 disables the inclusion of measurement data in the e-mail; any other
              value enables including measurement data in the e-mail.
            - ``ON`` enables the inclusion of measurement data in e-mail.
            - ``OFF`` disables the inclusion of measurement data in e-mail.
        """
        return self._measurement

    @property
    def numemails(self) -> EmailNumemails:
        """Return the ``EMail:NUMEMails`` command.

        Description:
            - This command sets or queries the number of e-mails you can send when Email on Event is
              armed, from 1 to 50. This limit is to restrict e-mail floods.

        Usage:
            - Using the ``.query()`` method will send the ``EMail:NUMEMails?`` query.
            - Using the ``.verify(value)`` method will send the ``EMail:NUMEMails?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``EMail:NUMEMails value`` command.

        SCPI Syntax:
            ```
            - EMail:NUMEMails {<NR1>}
            - EMail:NUMEMails?
            ```

        Info:
            - ``<NR1>`` can be 1 through 50. This is the number of e-mails you can send before the
              number of sent e-mails must be reset with the EMail RESet command.
        """
        return self._numemails

    @property
    def smtpport(self) -> EmailSmtpport:
        """Return the ``EMail:SMTPPort`` command.

        Description:
            - This command sets or queries the SMTP port number that the e-mail server uses if other
              than the default of 25.

        Usage:
            - Using the ``.query()`` method will send the ``EMail:SMTPPort?`` query.
            - Using the ``.verify(value)`` method will send the ``EMail:SMTPPort?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``EMail:SMTPPort value`` command.

        SCPI Syntax:
            ```
            - EMail:SMTPPort {<NR1>}
            - EMail:SMTPPort?
            ```

        Info:
            - ``<NR1>`` can be 1 through 65535. This number specifies the TCPIP port number.
        """
        return self._smtpport

    @property
    def smtpserver(self) -> EmailSmtpserver:
        """Return the ``EMail:SMTPServer`` command.

        Description:
            - This command sets or queries the address of the SMTP mail server.

        Usage:
            - Using the ``.query()`` method will send the ``EMail:SMTPServer?`` query.
            - Using the ``.verify(value)`` method will send the ``EMail:SMTPServer?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``EMail:SMTPServer value`` command.

        SCPI Syntax:
            ```
            - EMail:SMTPServer <QString>
            - EMail:SMTPServer?
            ```

        Info:
            - ``<QString>`` argument is the address of the SMTP mail server that will handle the
              mail service request. For example, 'smtp.tek.com'.
        """
        return self._smtpserver

    @property
    def status(self) -> EmailStatus:
        """Return the ``EMail:STATUS`` command.

        Description:
            - This query only command returns the status of the last e-mail you attempted to send.

        Usage:
            - Using the ``.query()`` method will send the ``EMail:STATUS?`` query.
            - Using the ``.verify(value)`` method will send the ``EMail:STATUS?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - EMail:STATUS?
            ```
        """
        return self._status

    @property
    def timeout(self) -> EmailTimeout:
        """Return the ``EMail:TIMEOut`` command.

        Description:
            - This command sets or queries the global timeout in seconds. The default is 30 seconds.
              You use this timeout for socket connections and might need to change it from the
              default on some networks.

        Usage:
            - Using the ``.query()`` method will send the ``EMail:TIMEOut?`` query.
            - Using the ``.verify(value)`` method will send the ``EMail:TIMEOut?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``EMail:TIMEOut value`` command.

        SCPI Syntax:
            ```
            - EMail:TIMEOut <NR1>
            - EMail:TIMEOut?
            ```

        Info:
            - ``<NR1>`` argument is the global timeout in seconds. The value for NR1 can be 0
              through 500.
        """
        return self._timeout

    @property
    def to(self) -> EmailTo:
        """Return the ``EMail:TO`` command.

        Description:
            - This command sets or queries the address of the recipient(s) of an e-mail.

        Usage:
            - Using the ``.query()`` method will send the ``EMail:TO?`` query.
            - Using the ``.verify(value)`` method will send the ``EMail:TO?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``EMail:TO value`` command.

        SCPI Syntax:
            ```
            - EMail:TO <QString>
            - EMail:TO?
            ```

        Info:
            - ``<QString>`` argument is the e-mail address of the recipient (or recipients).
              Multiple addresses are separated with semicolons (;). For example,
              'johndoe@tek.com;billsmith@tek.com' specifies that both johndoe and billsmith at
              tek.com will receive e-mail when the event occurs. The length of this string is
              limited to 252 characters.
        """
        return self._to

    @property
    def trigger(self) -> EmailTrigger:
        """Return the ``EMail:TRIGger`` command.

        Description:
            - This command sets or queries whether e-mail is sent when a trigger occurs. The e-mail
              is sent after the acquisition is complete.

        Usage:
            - Using the ``.query()`` method will send the ``EMail:TRIGger?`` query.
            - Using the ``.verify(value)`` method will send the ``EMail:TRIGger?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``EMail:TRIGger value`` command.

        SCPI Syntax:
            ```
            - EMail:TRIGger {<NR1>|ON|OFF}
            - EMail:TRIGger?
            ```

        Info:
            - ``<NR1>`` = 0 disables sending e-mail when a trigger occurs; any other value enables
              sending e-mail when a trigger occurs.
            - ``ON`` argument enables sending e-mail when a trigger occurs.
            - ``OFF`` disables sending e-mail when a trigger occurs.
        """
        return self._trigger

    @property
    def waveform(self) -> EmailWaveform:
        """Return the ``EMail:WAVEform`` command.

        Description:
            - This command sets or queries whether waveform data is included in e-mail. The
              waveforms to be included, the start and stop points (and, or, frames for Fast Frame
              mode), and the data formats are specified by the DATA and WFMOUTPRE commands within
              the Waveform Transfer Command group.

        Usage:
            - Using the ``.query()`` method will send the ``EMail:WAVEform?`` query.
            - Using the ``.verify(value)`` method will send the ``EMail:WAVEform?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``EMail:WAVEform value`` command.

        SCPI Syntax:
            ```
            - EMail:WAVEform {<NR1>|ON|OFF}
            - EMail:WAVEform?
            ```

        Info:
            - ``<NR1>`` = 0 disables the inclusion of waveform data in the e-mail; any other value
              enables including waveform data in the e-mail.
            - ``ON`` enables the inclusion of waveform data in e-mail.
            - ``OFF`` disables the inclusion of waveform data in e-mail.
        """
        return self._waveform



"""The autoset commands module.

These commands are used in the following models:
DPO4K, DPO4KB, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO4K, MSO4KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - AUTOSet {EXECute|UNDo}
    - AUTOSet:ENAble {OFF|ON|0|1}
    - AUTOSet:ENAble?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class AutosetEnable(SCPICmdWrite, SCPICmdRead):
    """The ``AUTOSet:ENAble`` command.

    Description:
        - Enables or disables the autoset feature. This is useful for classroom purposes where the
          instructor wants the students to achieve the desired instrument settings without the
          benefit of the autoset feature. This setting is not saved in setup files or SET? or
          ``*LRN?`` queries. The default state is 1 (autoset enabled).

    Usage:
        - Using the ``.query()`` method will send the ``AUTOSet:ENAble?`` query.
        - Using the ``.verify(value)`` method will send the ``AUTOSet:ENAble?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUTOSet:ENAble value`` command.

    SCPI Syntax:
        ```
        - AUTOSet:ENAble {OFF|ON|0|1}
        - AUTOSet:ENAble?
        ```

    Info:
        - ``OFF or 0`` disables autoset.
        - ``ON or 1`` enables autoset.
    """


class Autoset(SCPICmdWrite, SCPICmdRead):
    """The ``AUTOSet`` command.

    Description:
        - Sets the vertical, horizontal, and trigger controls of the oscilloscope to automatically
          acquire and display the selected waveform.

    Usage:
        - Using the ``.write(value)`` method will send the ``AUTOSet value`` command.

    SCPI Syntax:
        ```
        - AUTOSet {EXECute|UNDo}
        ```

    Info:
        - ``EXECute`` autosets the displayed waveform.
        - ``UNDo`` restores the oscilloscope settings to those present prior to the autoset
          execution.

    Properties:
        - ``.enable``: The ``AUTOSet:ENAble`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "AUTOSet") -> None:
        super().__init__(device, cmd_syntax)
        self._enable = AutosetEnable(device, f"{self._cmd_syntax}:ENAble")

    @property
    def enable(self) -> AutosetEnable:
        """Return the ``AUTOSet:ENAble`` command.

        Description:
            - Enables or disables the autoset feature. This is useful for classroom purposes where
              the instructor wants the students to achieve the desired instrument settings without
              the benefit of the autoset feature. This setting is not saved in setup files or SET?
              or ``*LRN?`` queries. The default state is 1 (autoset enabled).

        Usage:
            - Using the ``.query()`` method will send the ``AUTOSet:ENAble?`` query.
            - Using the ``.verify(value)`` method will send the ``AUTOSet:ENAble?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUTOSet:ENAble value`` command.

        SCPI Syntax:
            ```
            - AUTOSet:ENAble {OFF|ON|0|1}
            - AUTOSet:ENAble?
            ```

        Info:
            - ``OFF or 0`` disables autoset.
            - ``ON or 1`` enables autoset.
        """
        return self._enable



"""The abort commands module.

These commands are used in the following models:
AFG3K, AFG3KB, AFG3KC, AWG5K, AWG5KC, AWG7K, AWG7KC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - ABORt
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class Abort(SCPICmdWriteNoArguments):
    """The ``ABORt`` command.

    Description:
        - Initializes all the current trigger system parameters and resets all trigger sequences.

    Usage:
        - Using the ``.write()`` method will send the ``ABORt`` command.

    SCPI Syntax:
        ```
        - ABORt
        ```
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "ABORt") -> None:
        super().__init__(device, cmd_syntax)



"""The miscellaneous commands module.

These commands are used in the following models:
AFG3K, AFG3KB, AFG3KC, AWG5200, AWG5K, AWG5KC, AWG70KA, AWG70KB, AWG7K, AWG7KC, DPO2K, DPO2KB,
DPO4K, DPO4KB, DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7K, DPO7KC, DSA70KC, DSA70KD,
LPD6, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO2, MSO2K, MSO2KB, MSO4, MSO4B, MSO4K, MSO4KB, MSO5,
MSO5B, MSO5K, MSO5KB, MSO5LP, MSO6, MSO6B, MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - *IDN?
    - *TRG
    - *TST?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class Tst(SCPICmdRead):
    """The ``*TST`` command.

    Description:
        - Tests (self-test) the interface and returns a 0.

    Usage:
        - Using the ``.query()`` method will send the ``*TST?`` query.
        - Using the ``.verify(value)`` method will send the ``*TST?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - *TST?
        ```
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "*TST") -> None:
        super().__init__(device, cmd_syntax)


class Trg(SCPICmdWriteNoArguments):
    """The ``*TRG`` command.

    Description:
        - Performs a group execute trigger on commands defined by ``*DDT``.

    Usage:
        - Using the ``.write()`` method will send the ``*TRG`` command.

    SCPI Syntax:
        ```
        - *TRG
        ```
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "*TRG") -> None:
        super().__init__(device, cmd_syntax)


class Idn(SCPICmdRead):
    """The ``*IDN`` command.

    Description:
        - This query-only command returns the instrument identification code.

    Usage:
        - Using the ``.query()`` method will send the ``*IDN?`` query.
        - Using the ``.verify(value)`` method will send the ``*IDN?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - *IDN?
        ```
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "*IDN") -> None:
        super().__init__(device, cmd_syntax)



# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The timer commands module.

These commands are used in the following models:
SMU2601B, SMU2602B, SMU2604B, SMU2606B, SMU2611B, SMU2612B, SMU2614B, SMU2634B, SMU2635B, SMU2636B,
SMU2651A, SMU2657A, SS3706A

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:
    ```
    - timer.measure.t()
    - timer.reset()
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


class TimerMeasure(BaseTSPCmd):
    """The ``timer.measure`` command tree.

    Properties and methods:
        - ``.t()``: The ``timer.measure.t()`` function.
    """

    def t(self) -> str:
        """Run the ``timer.measure.t()`` function.

        Description:
            - This function measures the elapsed time since the timer was last reset.

        TSP Syntax:
            ```
            - timer.measure.t()
            ```

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.t())"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.t()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class Timer(BaseTSPCmd):
    """The ``timer`` command tree.

    Properties and methods:
        - ``.measure``: The ``timer.measure`` command tree.
        - ``.reset()``: The ``timer.reset()`` function.
    """

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "timer") -> None:
        super().__init__(device, cmd_syntax)
        self._measure = TimerMeasure(device, f"{self._cmd_syntax}.measure")

    @property
    def measure(self) -> TimerMeasure:
        """Return the ``timer.measure`` command tree.

        Sub-properties and sub-methods:
            - ``.t()``: The ``timer.measure.t()`` function.
        """
        return self._measure

    def reset(self) -> None:
        """Run the ``timer.reset()`` function.

        Description:
            - This function resets the timer to zero (0) seconds.

        TSP Syntax:
            ```
            - timer.reset()
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.reset()"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.reset()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""The ch commands module.

These commands are used in the following models:
MSO2

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - CH<x>:BANdwidth {<NR3>|FULl}
    - CH<x>:BANdwidth?
    - CH<x>:CLIPping?
    - CH<x>:COUPling {AC|DC|DCREJect}
    - CH<x>:COUPling?
    - CH<x>:DESKew <NR3>
    - CH<x>:DESKew?
    - CH<x>:DITHERrange <NR3>
    - CH<x>:DITHERrange?
    - CH<x>:INVert {ON|OFF|<NR1>}
    - CH<x>:INVert?
    - CH<x>:LABel:COLor <QString>
    - CH<x>:LABel:COLor?
    - CH<x>:LABel:FONT:BOLD {ON|OFF|<NR1>}
    - CH<x>:LABel:FONT:BOLD?
    - CH<x>:LABel:FONT:ITALic {ON|OFF|<NR1>}
    - CH<x>:LABel:FONT:ITALic?
    - CH<x>:LABel:FONT:SIZE <NR1>
    - CH<x>:LABel:FONT:SIZE?
    - CH<x>:LABel:FONT:TYPE <QString>
    - CH<x>:LABel:FONT:TYPE?
    - CH<x>:LABel:FONT:UNDERline {ON|OFF|<NR1>}
    - CH<x>:LABel:FONT:UNDERline?
    - CH<x>:LABel:NAMe <QString>
    - CH<x>:LABel:NAMe?
    - CH<x>:LABel:XPOS <NR3>
    - CH<x>:LABel:XPOS?
    - CH<x>:LABel:YPOS <NR3>
    - CH<x>:LABel:YPOS?
    - CH<x>:OFFSet <NR3>
    - CH<x>:OFFSet?
    - CH<x>:POSition <NR1>
    - CH<x>:POSition?
    - CH<x>:PROBEFunc:EXTAtten <NR3>
    - CH<x>:PROBEFunc:EXTAtten?
    - CH<x>:PROBEFunc:EXTDBatten <NR3>
    - CH<x>:PROBEFunc:EXTDBatten?
    - CH<x>:PROBEFunc:EXTUnits <QString>
    - CH<x>:PROBEFunc:EXTUnits:STATE {ON|OFF|<NR1>}
    - CH<x>:PROBEFunc:EXTUnits:STATE?
    - CH<x>:PROBEFunc:EXTUnits?
    - CH<x>:SCALERATio <NR2>
    - CH<x>:SCALERATio?
    - CH<x>:SCAle <NR3>
    - CH<x>:SCAle?
    - CH<x>:TERmination <NR3>
    - CH<x>:TERmination?
    - CH<x>:VTERm:BIAS <NR3>
    - CH<x>:VTERm:BIAS?
    - CH<x>?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite, ValidatedChannel

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class ChannelVtermBias(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:VTERm:BIAS`` command.

    Description:
        - Sets or queries the termination bias voltage for the specified channel (if control is
          available).

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:VTERm:BIAS?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:VTERm:BIAS?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:VTERm:BIAS value`` command.

    SCPI Syntax:
        ```
        - CH<x>:VTERm:BIAS <NR3>
        - CH<x>:VTERm:BIAS?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``<NR3>`` is the termination voltage.
    """


class ChannelVterm(SCPICmdRead):
    """The ``CH<x>:VTERm`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:VTERm?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:VTERm?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Info:
        - ``CH<x>`` is the channel number.

    Properties:
        - ``.bias``: The ``CH<x>:VTERm:BIAS`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._bias = ChannelVtermBias(device, f"{self._cmd_syntax}:BIAS")

    @property
    def bias(self) -> ChannelVtermBias:
        """Return the ``CH<x>:VTERm:BIAS`` command.

        Description:
            - Sets or queries the termination bias voltage for the specified channel (if control is
              available).

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:VTERm:BIAS?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:VTERm:BIAS?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:VTERm:BIAS value`` command.

        SCPI Syntax:
            ```
            - CH<x>:VTERm:BIAS <NR3>
            - CH<x>:VTERm:BIAS?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``<NR3>`` is the termination voltage.
        """
        return self._bias


class ChannelTermination(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:TERmination`` command.

    Description:
        - This command sets or queries the vertical termination for the specified analog channel.
          The channel is specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:TERmination?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:TERmination?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:TERmination value`` command.

    SCPI Syntax:
        ```
        - CH<x>:TERmination <NR3>
        - CH<x>:TERmination?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``<NR3>`` specifies the channel input resistance, which can be specified as 50 Ω or
          1,000,000 Ω.
    """


class ChannelScale(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:SCAle`` command.

    Description:
        - This command sets or returns the vertical scale for the specified analog channel. The
          channel is specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:SCAle?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:SCAle?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:SCAle value`` command.

    SCPI Syntax:
        ```
        - CH<x>:SCAle <NR3>
        - CH<x>:SCAle?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``<NR3>`` is the vertical scale for the specified analog channel.
    """


class ChannelScaleratio(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:SCALERATio`` command.

    Description:
        - This command sets or returns the scale ratio for the specified analog channel.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:SCALERATio?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:SCALERATio?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:SCALERATio value`` command.

    SCPI Syntax:
        ```
        - CH<x>:SCALERATio <NR2>
        - CH<x>:SCALERATio?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``<NR2>`` is the scale ratio for the specified analog channel.
    """


class ChannelProbefuncExtunitsState(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:PROBEFunc:EXTUnits:STATE`` command.

    Description:
        - This command sets or queries measure current status as ON or OFF. The channel is specified
          by x. If this command is set, the vertical scale is set to 'A', as it implies that the
          unit is measuring current from a voltage probe. When it is unset, the vertical scale is
          set to the value of ``CHX:PROBEFUNC:EXTUNITS`` ('V' or 'A').

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:PROBEFunc:EXTUnits:STATE?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:PROBEFunc:EXTUnits:STATE?``
          query and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:PROBEFunc:EXTUnits:STATE value``
          command.

    SCPI Syntax:
        ```
        - CH<x>:PROBEFunc:EXTUnits:STATE {ON|OFF|<NR1>}
        - CH<x>:PROBEFunc:EXTUnits:STATE?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``OFF`` argument turns current status off.
        - ``ON`` argument turns current status on.
        - ``<NR1>`` = 0 turns current status off; any other value turns current status on.
    """


class ChannelProbefuncExtunits(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:PROBEFunc:EXTUnits`` command.

    Description:
        - This command sets the unit of measurement for the external attenuator of the specified
          channel. The channel is specified by x. The alternate units are used if they are enabled.
          Use the ``CHX:PROBEFUNC:EXTUNITS:STATE`` command to enable or disable the alternate units.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:PROBEFunc:EXTUnits?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:PROBEFunc:EXTUnits?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:PROBEFunc:EXTUnits value``
          command.

    SCPI Syntax:
        ```
        - CH<x>:PROBEFunc:EXTUnits <QString>
        - CH<x>:PROBEFunc:EXTUnits?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``<QString>`` indicates the attenuation unit of measurement for the specified channel.

    Properties:
        - ``.state``: The ``CH<x>:PROBEFunc:EXTUnits:STATE`` command.
    """

    _WRAP_ARG_WITH_QUOTES = True

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._state = ChannelProbefuncExtunitsState(device, f"{self._cmd_syntax}:STATE")

    @property
    def state(self) -> ChannelProbefuncExtunitsState:
        """Return the ``CH<x>:PROBEFunc:EXTUnits:STATE`` command.

        Description:
            - This command sets or queries measure current status as ON or OFF. The channel is
              specified by x. If this command is set, the vertical scale is set to 'A', as it
              implies that the unit is measuring current from a voltage probe. When it is unset, the
              vertical scale is set to the value of ``CHX:PROBEFUNC:EXTUNITS`` ('V' or 'A').

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:PROBEFunc:EXTUnits:STATE?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:PROBEFunc:EXTUnits:STATE?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``CH<x>:PROBEFunc:EXTUnits:STATE value`` command.

        SCPI Syntax:
            ```
            - CH<x>:PROBEFunc:EXTUnits:STATE {ON|OFF|<NR1>}
            - CH<x>:PROBEFunc:EXTUnits:STATE?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``OFF`` argument turns current status off.
            - ``ON`` argument turns current status on.
            - ``<NR1>`` = 0 turns current status off; any other value turns current status on.
        """
        return self._state


class ChannelProbefuncExtdbatten(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:PROBEFunc:EXTDBatten`` command.

    Description:
        - This command sets or queries the input-output ratio (expressed in decibel units) of
          external attenuation or gain between the signal and the instrument input channels. The
          channel is specified by x. The query form of this command returns the user-specified
          attenuation in decibels.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:PROBEFunc:EXTDBatten?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:PROBEFunc:EXTDBatten?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:PROBEFunc:EXTDBatten value``
          command.

    SCPI Syntax:
        ```
        - CH<x>:PROBEFunc:EXTDBatten <NR3>
        - CH<x>:PROBEFunc:EXTDBatten?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``<NR3>`` is the attenuation value, which is specified in the range from -200.00 dB to
          200.00 dB.
    """


class ChannelProbefuncExtatten(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:PROBEFunc:EXTAtten`` command.

    Description:
        - This command is used to specify the attenuation value as a multiplier to the given scale
          factor on the specified channel. The channel is specified by x. The query form of this
          command returns the user-specified attenuation.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:PROBEFunc:EXTAtten?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:PROBEFunc:EXTAtten?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:PROBEFunc:EXTAtten value``
          command.

    SCPI Syntax:
        ```
        - CH<x>:PROBEFunc:EXTAtten <NR3>
        - CH<x>:PROBEFunc:EXTAtten?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``<NR3>`` is the attenuation value, which is specified as a multiplier in the range from
          1.00E-10 to 1.00E+10.
    """


class ChannelProbefunc(SCPICmdRead):
    """The ``CH<x>:PROBEFunc`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:PROBEFunc?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:PROBEFunc?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Info:
        - ``CH<x>`` is the channel number.

    Properties:
        - ``.extatten``: The ``CH<x>:PROBEFunc:EXTAtten`` command.
        - ``.extdbatten``: The ``CH<x>:PROBEFunc:EXTDBatten`` command.
        - ``.extunits``: The ``CH<x>:PROBEFunc:EXTUnits`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._extatten = ChannelProbefuncExtatten(device, f"{self._cmd_syntax}:EXTAtten")
        self._extdbatten = ChannelProbefuncExtdbatten(device, f"{self._cmd_syntax}:EXTDBatten")
        self._extunits = ChannelProbefuncExtunits(device, f"{self._cmd_syntax}:EXTUnits")

    @property
    def extatten(self) -> ChannelProbefuncExtatten:
        """Return the ``CH<x>:PROBEFunc:EXTAtten`` command.

        Description:
            - This command is used to specify the attenuation value as a multiplier to the given
              scale factor on the specified channel. The channel is specified by x. The query form
              of this command returns the user-specified attenuation.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:PROBEFunc:EXTAtten?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:PROBEFunc:EXTAtten?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:PROBEFunc:EXTAtten value``
              command.

        SCPI Syntax:
            ```
            - CH<x>:PROBEFunc:EXTAtten <NR3>
            - CH<x>:PROBEFunc:EXTAtten?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``<NR3>`` is the attenuation value, which is specified as a multiplier in the range
              from 1.00E-10 to 1.00E+10.
        """
        return self._extatten

    @property
    def extdbatten(self) -> ChannelProbefuncExtdbatten:
        """Return the ``CH<x>:PROBEFunc:EXTDBatten`` command.

        Description:
            - This command sets or queries the input-output ratio (expressed in decibel units) of
              external attenuation or gain between the signal and the instrument input channels. The
              channel is specified by x. The query form of this command returns the user-specified
              attenuation in decibels.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:PROBEFunc:EXTDBatten?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:PROBEFunc:EXTDBatten?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:PROBEFunc:EXTDBatten value``
              command.

        SCPI Syntax:
            ```
            - CH<x>:PROBEFunc:EXTDBatten <NR3>
            - CH<x>:PROBEFunc:EXTDBatten?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``<NR3>`` is the attenuation value, which is specified in the range from -200.00 dB to
              200.00 dB.
        """
        return self._extdbatten

    @property
    def extunits(self) -> ChannelProbefuncExtunits:
        """Return the ``CH<x>:PROBEFunc:EXTUnits`` command.

        Description:
            - This command sets the unit of measurement for the external attenuator of the specified
              channel. The channel is specified by x. The alternate units are used if they are
              enabled. Use the ``CHX:PROBEFUNC:EXTUNITS:STATE`` command to enable or disable the
              alternate units.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:PROBEFunc:EXTUnits?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:PROBEFunc:EXTUnits?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:PROBEFunc:EXTUnits value``
              command.

        SCPI Syntax:
            ```
            - CH<x>:PROBEFunc:EXTUnits <QString>
            - CH<x>:PROBEFunc:EXTUnits?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``<QString>`` indicates the attenuation unit of measurement for the specified channel.

        Sub-properties:
            - ``.state``: The ``CH<x>:PROBEFunc:EXTUnits:STATE`` command.
        """
        return self._extunits


class ChannelPosition(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:POSition`` command.

    Description:
        - This command sets or queries the vertical position for the specified analog channel.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:POSition?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:POSition?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:POSition value`` command.

    SCPI Syntax:
        ```
        - CH<x>:POSition <NR1>
        - CH<x>:POSition?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``<NR1>`` is the vertical position for the specified analog channel.
    """


class ChannelOffset(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:OFFSet`` command.

    Description:
        - This command sets or queries the vertical offset for the specified analog channel.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:OFFSet?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:OFFSet?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:OFFSet value`` command.

    SCPI Syntax:
        ```
        - CH<x>:OFFSet <NR3>
        - CH<x>:OFFSet?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``<NR3>`` is the offset value for the specified channel.
    """


class ChannelLabelYpos(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:LABel:YPOS`` command.

    Description:
        - This command sets or queries the Y-position of the specified channel label. The channel is
          specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:LABel:YPOS?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:LABel:YPOS?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:LABel:YPOS value`` command.

    SCPI Syntax:
        ```
        - CH<x>:LABel:YPOS <NR3>
        - CH<x>:LABel:YPOS?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``<NR3>`` is the location (in pixels) where the waveform label for the selected channel is
          displayed, relative to the baseline of the waveform. Positive values are above the
          baseline and negative values are below.
    """


class ChannelLabelXpos(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:LABel:XPOS`` command.

    Description:
        - This command sets or queries the X-position of the specified channel label. The channel is
          specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:LABel:XPOS?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:LABel:XPOS?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:LABel:XPOS value`` command.

    SCPI Syntax:
        ```
        - CH<x>:LABel:XPOS <NR3>
        - CH<x>:LABel:XPOS?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``<NR3>`` is the location (in pixels) where the waveform label for the selected channel is
          displayed, relative to the left edge of the screen.
    """


class ChannelLabelName(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:LABel:NAMe`` command.

    Description:
        - This command sets or queries the label attached to the displayed waveform for the
          specified channel. The channel is specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:LABel:NAMe?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:LABel:NAMe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:LABel:NAMe value`` command.

    SCPI Syntax:
        ```
        - CH<x>:LABel:NAMe <QString>
        - CH<x>:LABel:NAMe?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``<QString>`` is an alphanumeric character string, ranging from 1 through 32 characters in
          length.
    """

    _WRAP_ARG_WITH_QUOTES = True


class ChannelLabelFontUnderline(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:LABel:FONT:UNDERline`` command.

    Description:
        - This command sets or queries the underline state of the specified channel label. The
          channel is specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:LABel:FONT:UNDERline?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:LABel:FONT:UNDERline?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:LABel:FONT:UNDERline value``
          command.

    SCPI Syntax:
        ```
        - CH<x>:LABel:FONT:UNDERline {ON|OFF|<NR1>}
        - CH<x>:LABel:FONT:UNDERline?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``OFF`` argument turns off underlined font.
        - ``ON`` argument turns on underlined font.
        - ``<NR1>`` = 0 turns off underlined font; any other value turns on underlined font.
    """


class ChannelLabelFontType(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:LABel:FONT:TYPE`` command.

    Description:
        - This command sets or queries the font type of the specified channel label, such as Arial
          or Times New Roman. The channel is specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:LABel:FONT:TYPE?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:LABel:FONT:TYPE?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:LABel:FONT:TYPE value`` command.

    SCPI Syntax:
        ```
        - CH<x>:LABel:FONT:TYPE <QString>
        - CH<x>:LABel:FONT:TYPE?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``<QString>`` is the specified font type.
    """

    _WRAP_ARG_WITH_QUOTES = True


class ChannelLabelFontSize(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:LABel:FONT:SIZE`` command.

    Description:
        - This command sets or queries the font size of the specified channel label. The channel is
          specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:LABel:FONT:SIZE?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:LABel:FONT:SIZE?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:LABel:FONT:SIZE value`` command.

    SCPI Syntax:
        ```
        - CH<x>:LABel:FONT:SIZE <NR1>
        - CH<x>:LABel:FONT:SIZE?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``<NR1>`` is the font size.
    """


class ChannelLabelFontItalic(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:LABel:FONT:ITALic`` command.

    Description:
        - This command sets or queries the italic state of the specified channel label. The channel
          is specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:LABel:FONT:ITALic?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:LABel:FONT:ITALic?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:LABel:FONT:ITALic value``
          command.

    SCPI Syntax:
        ```
        - CH<x>:LABel:FONT:ITALic {ON|OFF|<NR1>}
        - CH<x>:LABel:FONT:ITALic?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``OFF`` argument turns off italic font.
        - ``ON`` argument turns on italic font.
        - ``<NR1>`` = 0 turns off italic font; any other value turns on italic font.
    """


class ChannelLabelFontBold(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:LABel:FONT:BOLD`` command.

    Description:
        - This command sets or queries the bold state of the specified channel label. The channel is
          specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:LABel:FONT:BOLD?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:LABel:FONT:BOLD?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:LABel:FONT:BOLD value`` command.

    SCPI Syntax:
        ```
        - CH<x>:LABel:FONT:BOLD {ON|OFF|<NR1>}
        - CH<x>:LABel:FONT:BOLD?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``OFF`` argument turns off bold font.
        - ``ON`` argument turns on bold font.
        - ``<NR1>`` = 0 turns off bold font; any other value turns on bold font.
    """


class ChannelLabelFont(SCPICmdRead):
    """The ``CH<x>:LABel:FONT`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:LABel:FONT?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:LABel:FONT?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Info:
        - ``CH<x>`` is the channel number.

    Properties:
        - ``.bold``: The ``CH<x>:LABel:FONT:BOLD`` command.
        - ``.italic``: The ``CH<x>:LABel:FONT:ITALic`` command.
        - ``.size``: The ``CH<x>:LABel:FONT:SIZE`` command.
        - ``.type``: The ``CH<x>:LABel:FONT:TYPE`` command.
        - ``.underline``: The ``CH<x>:LABel:FONT:UNDERline`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._bold = ChannelLabelFontBold(device, f"{self._cmd_syntax}:BOLD")
        self._italic = ChannelLabelFontItalic(device, f"{self._cmd_syntax}:ITALic")
        self._size = ChannelLabelFontSize(device, f"{self._cmd_syntax}:SIZE")
        self._type = ChannelLabelFontType(device, f"{self._cmd_syntax}:TYPE")
        self._underline = ChannelLabelFontUnderline(device, f"{self._cmd_syntax}:UNDERline")

    @property
    def bold(self) -> ChannelLabelFontBold:
        """Return the ``CH<x>:LABel:FONT:BOLD`` command.

        Description:
            - This command sets or queries the bold state of the specified channel label. The
              channel is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:LABel:FONT:BOLD?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:LABel:FONT:BOLD?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:LABel:FONT:BOLD value``
              command.

        SCPI Syntax:
            ```
            - CH<x>:LABel:FONT:BOLD {ON|OFF|<NR1>}
            - CH<x>:LABel:FONT:BOLD?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``OFF`` argument turns off bold font.
            - ``ON`` argument turns on bold font.
            - ``<NR1>`` = 0 turns off bold font; any other value turns on bold font.
        """
        return self._bold

    @property
    def italic(self) -> ChannelLabelFontItalic:
        """Return the ``CH<x>:LABel:FONT:ITALic`` command.

        Description:
            - This command sets or queries the italic state of the specified channel label. The
              channel is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:LABel:FONT:ITALic?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:LABel:FONT:ITALic?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:LABel:FONT:ITALic value``
              command.

        SCPI Syntax:
            ```
            - CH<x>:LABel:FONT:ITALic {ON|OFF|<NR1>}
            - CH<x>:LABel:FONT:ITALic?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``OFF`` argument turns off italic font.
            - ``ON`` argument turns on italic font.
            - ``<NR1>`` = 0 turns off italic font; any other value turns on italic font.
        """
        return self._italic

    @property
    def size(self) -> ChannelLabelFontSize:
        """Return the ``CH<x>:LABel:FONT:SIZE`` command.

        Description:
            - This command sets or queries the font size of the specified channel label. The channel
              is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:LABel:FONT:SIZE?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:LABel:FONT:SIZE?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:LABel:FONT:SIZE value``
              command.

        SCPI Syntax:
            ```
            - CH<x>:LABel:FONT:SIZE <NR1>
            - CH<x>:LABel:FONT:SIZE?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``<NR1>`` is the font size.
        """
        return self._size

    @property
    def type(self) -> ChannelLabelFontType:
        """Return the ``CH<x>:LABel:FONT:TYPE`` command.

        Description:
            - This command sets or queries the font type of the specified channel label, such as
              Arial or Times New Roman. The channel is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:LABel:FONT:TYPE?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:LABel:FONT:TYPE?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:LABel:FONT:TYPE value``
              command.

        SCPI Syntax:
            ```
            - CH<x>:LABel:FONT:TYPE <QString>
            - CH<x>:LABel:FONT:TYPE?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``<QString>`` is the specified font type.
        """
        return self._type

    @property
    def underline(self) -> ChannelLabelFontUnderline:
        """Return the ``CH<x>:LABel:FONT:UNDERline`` command.

        Description:
            - This command sets or queries the underline state of the specified channel label. The
              channel is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:LABel:FONT:UNDERline?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:LABel:FONT:UNDERline?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:LABel:FONT:UNDERline value``
              command.

        SCPI Syntax:
            ```
            - CH<x>:LABel:FONT:UNDERline {ON|OFF|<NR1>}
            - CH<x>:LABel:FONT:UNDERline?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``OFF`` argument turns off underlined font.
            - ``ON`` argument turns on underlined font.
            - ``<NR1>`` = 0 turns off underlined font; any other value turns on underlined font.
        """
        return self._underline


class ChannelLabelColor(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:LABel:COLor`` command.

    Description:
        - This command sets or queries the color of the specified channel label. The channel is
          specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:LABel:COLor?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:LABel:COLor?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:LABel:COLor value`` command.

    SCPI Syntax:
        ```
        - CH<x>:LABel:COLor <QString>
        - CH<x>:LABel:COLor?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``<QString>`` is the label color. To return the color to the default color, send an empty
          string as in this example: ``CH5:LABEL:COLOR`` ''.
    """

    _WRAP_ARG_WITH_QUOTES = True


class ChannelLabel(SCPICmdRead):
    """The ``CH<x>:LABel`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:LABel?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:LABel?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Info:
        - ``CH<x>`` is the channel number.

    Properties:
        - ``.color``: The ``CH<x>:LABel:COLor`` command.
        - ``.font``: The ``CH<x>:LABel:FONT`` command tree.
        - ``.name``: The ``CH<x>:LABel:NAMe`` command.
        - ``.xpos``: The ``CH<x>:LABel:XPOS`` command.
        - ``.ypos``: The ``CH<x>:LABel:YPOS`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._color = ChannelLabelColor(device, f"{self._cmd_syntax}:COLor")
        self._font = ChannelLabelFont(device, f"{self._cmd_syntax}:FONT")
        self._name = ChannelLabelName(device, f"{self._cmd_syntax}:NAMe")
        self._xpos = ChannelLabelXpos(device, f"{self._cmd_syntax}:XPOS")
        self._ypos = ChannelLabelYpos(device, f"{self._cmd_syntax}:YPOS")

    @property
    def color(self) -> ChannelLabelColor:
        """Return the ``CH<x>:LABel:COLor`` command.

        Description:
            - This command sets or queries the color of the specified channel label. The channel is
              specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:LABel:COLor?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:LABel:COLor?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:LABel:COLor value`` command.

        SCPI Syntax:
            ```
            - CH<x>:LABel:COLor <QString>
            - CH<x>:LABel:COLor?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``<QString>`` is the label color. To return the color to the default color, send an
              empty string as in this example: ``CH5:LABEL:COLOR`` ''.
        """
        return self._color

    @property
    def font(self) -> ChannelLabelFont:
        """Return the ``CH<x>:LABel:FONT`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:LABel:FONT?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:LABel:FONT?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Info:
            - ``CH<x>`` is the channel number.

        Sub-properties:
            - ``.bold``: The ``CH<x>:LABel:FONT:BOLD`` command.
            - ``.italic``: The ``CH<x>:LABel:FONT:ITALic`` command.
            - ``.size``: The ``CH<x>:LABel:FONT:SIZE`` command.
            - ``.type``: The ``CH<x>:LABel:FONT:TYPE`` command.
            - ``.underline``: The ``CH<x>:LABel:FONT:UNDERline`` command.
        """
        return self._font

    @property
    def name(self) -> ChannelLabelName:
        """Return the ``CH<x>:LABel:NAMe`` command.

        Description:
            - This command sets or queries the label attached to the displayed waveform for the
              specified channel. The channel is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:LABel:NAMe?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:LABel:NAMe?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:LABel:NAMe value`` command.

        SCPI Syntax:
            ```
            - CH<x>:LABel:NAMe <QString>
            - CH<x>:LABel:NAMe?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``<QString>`` is an alphanumeric character string, ranging from 1 through 32
              characters in length.
        """
        return self._name

    @property
    def xpos(self) -> ChannelLabelXpos:
        """Return the ``CH<x>:LABel:XPOS`` command.

        Description:
            - This command sets or queries the X-position of the specified channel label. The
              channel is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:LABel:XPOS?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:LABel:XPOS?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:LABel:XPOS value`` command.

        SCPI Syntax:
            ```
            - CH<x>:LABel:XPOS <NR3>
            - CH<x>:LABel:XPOS?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``<NR3>`` is the location (in pixels) where the waveform label for the selected
              channel is displayed, relative to the left edge of the screen.
        """
        return self._xpos

    @property
    def ypos(self) -> ChannelLabelYpos:
        """Return the ``CH<x>:LABel:YPOS`` command.

        Description:
            - This command sets or queries the Y-position of the specified channel label. The
              channel is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:LABel:YPOS?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:LABel:YPOS?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:LABel:YPOS value`` command.

        SCPI Syntax:
            ```
            - CH<x>:LABel:YPOS <NR3>
            - CH<x>:LABel:YPOS?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``<NR3>`` is the location (in pixels) where the waveform label for the selected
              channel is displayed, relative to the baseline of the waveform. Positive values are
              above the baseline and negative values are below.
        """
        return self._ypos


class ChannelInvert(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:INVert`` command.

    Description:
        - This command sets or queries invert state of the specified channel. The channel is
          specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:INVert?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:INVert?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:INVert value`` command.

    SCPI Syntax:
        ```
        - CH<x>:INVert {ON|OFF|<NR1>}
        - CH<x>:INVert?
        ```

    Info:
        - ``OFF`` turns off the channel invert.
        - ``ON`` turns on the channel invert.
        - ``<NR1>`` 0 turns off the channel invert; any other value turns on the channel invert.
    """


class ChannelDitherrange(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:DITHERrange`` command.

    Description:
        - This command sets or returns the amount of dithering for the specified analog channel. The
          channel is specified by x. The amount of dithering is a percentage of full scale (10 times
          volts/division). Note: Setting this value to 0.0 for any unused channels may slightly
          improve performance.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:DITHERrange?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:DITHERrange?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:DITHERrange value`` command.

    SCPI Syntax:
        ```
        - CH<x>:DITHERrange <NR3>
        - CH<x>:DITHERrange?
        ```

    Info:
        - ``<NR3>`` is the amount of dithering as a percentage of full scale. Must be between 0.0
          and 100.0 and 0.0 disables dithering.
    """


class ChannelDeskew(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:DESKew`` command.

    Description:
        - This command sets or queries the horizontal deskew time for the specified channel. The
          channel is specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:DESKew?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:DESKew?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:DESKew value`` command.

    SCPI Syntax:
        ```
        - CH<x>:DESKew <NR3>
        - CH<x>:DESKew?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``<NR3>`` is the deskew time for this channel, ranging from -125 ns to +125 ns with a
          resolution of 40 ps. Out-of-range values are clipped.
    """


class ChannelCoupling(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:COUPling`` command.

    Description:
        - This command sets or queries the input coupling setting for the specified analog channel.
          The channel is specified by x. The available arguments depend on the attached accessories.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:COUPling?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:COUPling?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:COUPling value`` command.

    SCPI Syntax:
        ```
        - CH<x>:COUPling {AC|DC|DCREJect}
        - CH<x>:COUPling?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``AC`` sets the specified channel to AC coupling.
        - ``DC`` sets the specified channel to DC coupling.
        - ``DCREJect`` sets DC Reject coupling when probes are attached that support that feature.
    """


class ChannelClipping(SCPICmdRead):
    """The ``CH<x>:CLIPping`` command.

    Description:
        - Queries whether the specified channel's input signal is clipping (exceeding) the channel
          A/D converter range. The channel is specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:CLIPping?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:CLIPping?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - CH<x>:CLIPping?
        ```
    """


class ChannelBandwidth(SCPICmdWrite, SCPICmdRead):
    """The ``CH<x>:BANdwidth`` command.

    Description:
        - This command sets or queries the selectable low-pass bandwidth limit filter of the
          specified channel. The channel is specified by x. The query form of this command always
          returns the approximate realized bandwidth of the channel. Available arguments depend upon
          the instrument and the attached accessories.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>:BANdwidth?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>:BANdwidth?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CH<x>:BANdwidth value`` command.

    SCPI Syntax:
        ```
        - CH<x>:BANdwidth {<NR3>|FULl}
        - CH<x>:BANdwidth?
        ```

    Info:
        - ``CH<x>`` is the channel number.
        - ``<NR3>`` is the desired bandwidth. The instrument rounds this value to an available
          bandwidth using geometric rounding and then uses this value to set the upper bandwidth.
        - ``FULl`` disables any optional bandwidth limiting. The specified channel operates at its
          maximum bandwidth.
    """


#  pylint: disable=too-many-instance-attributes
class Channel(ValidatedChannel, SCPICmdRead):
    """The ``CH<x>`` command.

    Description:
        - This query-only command returns the vertical parameters for the specified channel. The
          channel is specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``CH<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``CH<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - CH<x>?
        ```

    Properties:
        - ``.bandwidth``: The ``CH<x>:BANdwidth`` command.
        - ``.clipping``: The ``CH<x>:CLIPping`` command.
        - ``.coupling``: The ``CH<x>:COUPling`` command.
        - ``.deskew``: The ``CH<x>:DESKew`` command.
        - ``.ditherrange``: The ``CH<x>:DITHERrange`` command.
        - ``.invert``: The ``CH<x>:INVert`` command.
        - ``.label``: The ``CH<x>:LABel`` command tree.
        - ``.offset``: The ``CH<x>:OFFSet`` command.
        - ``.position``: The ``CH<x>:POSition`` command.
        - ``.probefunc``: The ``CH<x>:PROBEFunc`` command tree.
        - ``.scaleratio``: The ``CH<x>:SCALERATio`` command.
        - ``.scale``: The ``CH<x>:SCAle`` command.
        - ``.termination``: The ``CH<x>:TERmination`` command.
        - ``.vterm``: The ``CH<x>:VTERm`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "CH<x>") -> None:
        super().__init__(device, cmd_syntax)
        self._bandwidth = ChannelBandwidth(device, f"{self._cmd_syntax}:BANdwidth")
        self._clipping = ChannelClipping(device, f"{self._cmd_syntax}:CLIPping")
        self._coupling = ChannelCoupling(device, f"{self._cmd_syntax}:COUPling")
        self._deskew = ChannelDeskew(device, f"{self._cmd_syntax}:DESKew")
        self._ditherrange = ChannelDitherrange(device, f"{self._cmd_syntax}:DITHERrange")
        self._invert = ChannelInvert(device, f"{self._cmd_syntax}:INVert")
        self._label = ChannelLabel(device, f"{self._cmd_syntax}:LABel")
        self._offset = ChannelOffset(device, f"{self._cmd_syntax}:OFFSet")
        self._position = ChannelPosition(device, f"{self._cmd_syntax}:POSition")
        self._probefunc = ChannelProbefunc(device, f"{self._cmd_syntax}:PROBEFunc")
        self._scaleratio = ChannelScaleratio(device, f"{self._cmd_syntax}:SCALERATio")
        self._scale = ChannelScale(device, f"{self._cmd_syntax}:SCAle")
        self._termination = ChannelTermination(device, f"{self._cmd_syntax}:TERmination")
        self._vterm = ChannelVterm(device, f"{self._cmd_syntax}:VTERm")

    @property
    def bandwidth(self) -> ChannelBandwidth:
        """Return the ``CH<x>:BANdwidth`` command.

        Description:
            - This command sets or queries the selectable low-pass bandwidth limit filter of the
              specified channel. The channel is specified by x. The query form of this command
              always returns the approximate realized bandwidth of the channel. Available arguments
              depend upon the instrument and the attached accessories.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:BANdwidth?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:BANdwidth?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:BANdwidth value`` command.

        SCPI Syntax:
            ```
            - CH<x>:BANdwidth {<NR3>|FULl}
            - CH<x>:BANdwidth?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``<NR3>`` is the desired bandwidth. The instrument rounds this value to an available
              bandwidth using geometric rounding and then uses this value to set the upper
              bandwidth.
            - ``FULl`` disables any optional bandwidth limiting. The specified channel operates at
              its maximum bandwidth.
        """
        return self._bandwidth

    @property
    def clipping(self) -> ChannelClipping:
        """Return the ``CH<x>:CLIPping`` command.

        Description:
            - Queries whether the specified channel's input signal is clipping (exceeding) the
              channel A/D converter range. The channel is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:CLIPping?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:CLIPping?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - CH<x>:CLIPping?
            ```
        """
        return self._clipping

    @property
    def coupling(self) -> ChannelCoupling:
        """Return the ``CH<x>:COUPling`` command.

        Description:
            - This command sets or queries the input coupling setting for the specified analog
              channel. The channel is specified by x. The available arguments depend on the attached
              accessories.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:COUPling?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:COUPling?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:COUPling value`` command.

        SCPI Syntax:
            ```
            - CH<x>:COUPling {AC|DC|DCREJect}
            - CH<x>:COUPling?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``AC`` sets the specified channel to AC coupling.
            - ``DC`` sets the specified channel to DC coupling.
            - ``DCREJect`` sets DC Reject coupling when probes are attached that support that
              feature.
        """
        return self._coupling

    @property
    def deskew(self) -> ChannelDeskew:
        """Return the ``CH<x>:DESKew`` command.

        Description:
            - This command sets or queries the horizontal deskew time for the specified channel. The
              channel is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:DESKew?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:DESKew?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:DESKew value`` command.

        SCPI Syntax:
            ```
            - CH<x>:DESKew <NR3>
            - CH<x>:DESKew?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``<NR3>`` is the deskew time for this channel, ranging from -125 ns to +125 ns with a
              resolution of 40 ps. Out-of-range values are clipped.
        """
        return self._deskew

    @property
    def ditherrange(self) -> ChannelDitherrange:
        """Return the ``CH<x>:DITHERrange`` command.

        Description:
            - This command sets or returns the amount of dithering for the specified analog channel.
              The channel is specified by x. The amount of dithering is a percentage of full scale
              (10 times volts/division). Note: Setting this value to 0.0 for any unused channels may
              slightly improve performance.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:DITHERrange?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:DITHERrange?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:DITHERrange value`` command.

        SCPI Syntax:
            ```
            - CH<x>:DITHERrange <NR3>
            - CH<x>:DITHERrange?
            ```

        Info:
            - ``<NR3>`` is the amount of dithering as a percentage of full scale. Must be between
              0.0 and 100.0 and 0.0 disables dithering.
        """
        return self._ditherrange

    @property
    def invert(self) -> ChannelInvert:
        """Return the ``CH<x>:INVert`` command.

        Description:
            - This command sets or queries invert state of the specified channel. The channel is
              specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:INVert?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:INVert?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:INVert value`` command.

        SCPI Syntax:
            ```
            - CH<x>:INVert {ON|OFF|<NR1>}
            - CH<x>:INVert?
            ```

        Info:
            - ``OFF`` turns off the channel invert.
            - ``ON`` turns on the channel invert.
            - ``<NR1>`` 0 turns off the channel invert; any other value turns on the channel invert.
        """
        return self._invert

    @property
    def label(self) -> ChannelLabel:
        """Return the ``CH<x>:LABel`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:LABel?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:LABel?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Info:
            - ``CH<x>`` is the channel number.

        Sub-properties:
            - ``.color``: The ``CH<x>:LABel:COLor`` command.
            - ``.font``: The ``CH<x>:LABel:FONT`` command tree.
            - ``.name``: The ``CH<x>:LABel:NAMe`` command.
            - ``.xpos``: The ``CH<x>:LABel:XPOS`` command.
            - ``.ypos``: The ``CH<x>:LABel:YPOS`` command.
        """
        return self._label

    @property
    def offset(self) -> ChannelOffset:
        """Return the ``CH<x>:OFFSet`` command.

        Description:
            - This command sets or queries the vertical offset for the specified analog channel.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:OFFSet?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:OFFSet?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:OFFSet value`` command.

        SCPI Syntax:
            ```
            - CH<x>:OFFSet <NR3>
            - CH<x>:OFFSet?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``<NR3>`` is the offset value for the specified channel.
        """
        return self._offset

    @property
    def position(self) -> ChannelPosition:
        """Return the ``CH<x>:POSition`` command.

        Description:
            - This command sets or queries the vertical position for the specified analog channel.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:POSition?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:POSition?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:POSition value`` command.

        SCPI Syntax:
            ```
            - CH<x>:POSition <NR1>
            - CH<x>:POSition?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``<NR1>`` is the vertical position for the specified analog channel.
        """
        return self._position

    @property
    def probefunc(self) -> ChannelProbefunc:
        """Return the ``CH<x>:PROBEFunc`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:PROBEFunc?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:PROBEFunc?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Info:
            - ``CH<x>`` is the channel number.

        Sub-properties:
            - ``.extatten``: The ``CH<x>:PROBEFunc:EXTAtten`` command.
            - ``.extdbatten``: The ``CH<x>:PROBEFunc:EXTDBatten`` command.
            - ``.extunits``: The ``CH<x>:PROBEFunc:EXTUnits`` command.
        """
        return self._probefunc

    @property
    def scaleratio(self) -> ChannelScaleratio:
        """Return the ``CH<x>:SCALERATio`` command.

        Description:
            - This command sets or returns the scale ratio for the specified analog channel.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:SCALERATio?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:SCALERATio?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:SCALERATio value`` command.

        SCPI Syntax:
            ```
            - CH<x>:SCALERATio <NR2>
            - CH<x>:SCALERATio?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``<NR2>`` is the scale ratio for the specified analog channel.
        """
        return self._scaleratio

    @property
    def scale(self) -> ChannelScale:
        """Return the ``CH<x>:SCAle`` command.

        Description:
            - This command sets or returns the vertical scale for the specified analog channel. The
              channel is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:SCAle?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:SCAle?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:SCAle value`` command.

        SCPI Syntax:
            ```
            - CH<x>:SCAle <NR3>
            - CH<x>:SCAle?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``<NR3>`` is the vertical scale for the specified analog channel.
        """
        return self._scale

    @property
    def termination(self) -> ChannelTermination:
        """Return the ``CH<x>:TERmination`` command.

        Description:
            - This command sets or queries the vertical termination for the specified analog
              channel. The channel is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:TERmination?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:TERmination?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CH<x>:TERmination value`` command.

        SCPI Syntax:
            ```
            - CH<x>:TERmination <NR3>
            - CH<x>:TERmination?
            ```

        Info:
            - ``CH<x>`` is the channel number.
            - ``<NR3>`` specifies the channel input resistance, which can be specified as 50 Ω or
              1,000,000 Ω.
        """
        return self._termination

    @property
    def vterm(self) -> ChannelVterm:
        """Return the ``CH<x>:VTERm`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``CH<x>:VTERm?`` query.
            - Using the ``.verify(value)`` method will send the ``CH<x>:VTERm?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Info:
            - ``CH<x>`` is the channel number.

        Sub-properties:
            - ``.bias``: The ``CH<x>:VTERm:BIAS`` command.
        """
        return self._vterm
